#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>

#define DPK_MINOR           68 /* random */
#define DPK_MINOR_RATELIMIT 69 /* random too */
#define DPK_BUF 1023
static char localbuf[DPK_BUF+1];

ssize_t dpk_write(struct file *filp, const char *ubuff,
		  size_t count, loff_t *offp)
{
	int c;

	if (count > DPK_BUF) count = DPK_BUF;
	if (copy_from_user(localbuf, ubuff, count))
		return -EFAULT;
	c = count;
	/* remove trailing newline, if any: we add it later */
	if (localbuf[c-1] == '\n') c--;
	localbuf[c] = '\0';
	if (MINOR(filp->f_dentry->d_inode->i_rdev) == DPK_MINOR
	    || printk_ratelimit() )
		printk("%s\n", localbuf);
	return count;
}

struct file_operations dpk_fops = {
	.owner = THIS_MODULE,
	.write =    dpk_write,
};

struct miscdevice dpk_misc = {
	.minor = DPK_MINOR,
	.name = "printk",
	.fops = &dpk_fops,
};
struct miscdevice dpk_misc_r = {
	.minor = DPK_MINOR_RATELIMIT,
	.name = "printk_ratelimit",
	.fops = &dpk_fops,
};

int dpk_init(void)
{
	misc_register(&dpk_misc);
	misc_register(&dpk_misc_r);
	return 0;
}

void dpk_exit(void)
{
    misc_deregister(&dpk_misc);
    misc_deregister(&dpk_misc_r);
}

module_init(dpk_init);
module_exit(dpk_exit);


